#!/usr/bin/perl -w
use strict;
use blib;

package MyWidget;
use TQt;
use TQt::isa qw(TQt::VBox);

sub NEW {
    shift->SUPER::NEW(@_);

    my $quit = TQt::PushButton("Quit", this, "quit");
    $quit->setFont(TQt::Font("Times", 18, &TQt::Font::Bold));

    TQt::app->connect($quit, TQ_SIGNAL('clicked()'), TQ_SLOT('quit()'));

    my $lcd = TQt::LCDNumber(2, this, "lcd");

    my $slider = TQt::Slider(&Horizontal, this, "slider");
    $slider->setRange(0, 99);
    $slider->setValue(0);

    $lcd->connect($slider, TQ_SIGNAL('valueChanged(int)'), TQ_SLOT('display(int)'));
}

package main;
use MyWidget;

my $a = TQt::Application(\@ARGV);

my $w = MyWidget;
$a->setMainWidget($w);
$w->show;
exit $a->exec;
