#!/usr/bin/perl -w
use strict;
use blib;
use TQt;

my $a = TQt::Application(\@ARGV);

my $quit = TQt::PushButton("Quit", undef);
$quit->resize(75, 30);
$quit->setFont(TQt::Font("Times", 18, &TQt::Font::Bold));

$a->connect($quit, TQ_SIGNAL('clicked()'), TQ_SLOT('quit()'));

$a->setMainWidget($quit);
$quit->show;
exit $a->exec;
