BEGIN{ chdir("PerlTQt/t") or die "couldn't chdir to PerlTQt: $!\n" }   
END{ chdir("../..") or die "couldn't chdir to ../..\n" }   

use blib;
use Test::Harness;

$Test::Harness::switches = "-w -Mblib=../blib";

@t = glob("*.t");

runtests(@t);