#!/usr/bin/perl -w
use strict;
use blib;
use TQt;

my $a = TQt::Application(\@ARGV);

my $hello = TQt::PushButton("Hello World!", undef);
$hello->resize(100, 30);

$a->setMainWidget($hello);
$hello->show;
exit $a->exec;
