#!/usr/bin/perl -w
use strict;
use TQt;
use ButtonsGroups;

my $a = TQt::Application(\@ARGV);

my $buttonsgroups = ButtonsGroups;
$buttonsgroups->resize(500, 250);
$buttonsgroups->setCaption("PerlTQt Example - Buttongroups");
$a->setMainWidget($buttonsgroups);
$buttonsgroups->show;
exit $a->exec;
