BEGIN { print "1..3\n" }

use TQt;

$a=0;

# testing if the TQt::Application ctor works

eval { $a=TQt::Application(\@ARGV) };

print +$@ ? "not ok\n" : "ok 1\n";

# testing wether the global object is properly setup

eval { TQt::app()->libraryPaths() };

print +$@ ? "not ok\n" : "ok 2\n";

# one second test of the event loop

TQt::Timer::singleShot( 300, TQt::app(), TQ_SLOT "quit()" );

print TQt::app()->exec ? "not ok\n" : "ok 3\n";
