package TQt::GlobalSpace;
use strict;
require TQt;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT;
our $allMeth = TQt::_internal::findAllMethods( TQt::_internal::idClass("TQGlobalSpace") );
no strict 'refs';

for my $proto( keys %$allMeth )
{
    next if $proto =~ /operator\W/; # skip operators
    $proto =~ s/[\#\$\?]+$//;
    *{ $proto } = sub
                   {
                        $TQt::_internal::autoload::AUTOLOAD = "TQt::GlobalSpace\::$proto";
                        goto &TQt::GlobalSpace::AUTOLOAD
                   } unless defined &$proto;
     push @EXPORT, $proto;
}

our %EXPORT_TAGS = ( "all" => [@EXPORT] );

1;