#!/usr/bin/perl -w
use strict;
use TQt;
use DigitalClock;

my $a = TQt::Application(\@ARGV);
my $clock = DigitalClock;
$clock->resize(170, 80);
$a->setMainWidget($clock);
$clock->setCaption("PerlTQt Example - Digital Clock");
$clock->show;
exit $a->exec;
