#!/usr/bin/perl -w
use strict;
use TQt;
use AnalogClock;

my $a = TQt::Application(\@ARGV);
my $clock = AnalogClock;
$clock->setAutoMask(1) if @ARGV and $ARGV[0] eq '-transparent';
$clock->resize(100, 100);
$a->setMainWidget($clock);
$clock->setCaption("PerlTQt example - Analog Clock");
$clock->show;
exit $a->exec;
