package TQt::enumerations;
#
# Proposed usage:
#
# package MyWidget;
#
# use TQt::enumerations MyInfo => {
#     Foo => 1,
#     Bar => 10,
#     Baz => 64
# };
#
# use TQt::enumerations MyInfo => [qw(Foo Bar Baz)];
#
1;
