package TQt::properties;
#
# Proposed usage:
#
# use TQt::properties foo => {
#     TYPE => 'bool',
#     READ => 'getFoo',
#     WRITE => 'setFoo',
#     STORED => 0,
#     RESET => 'unsetFoo',
#     DESIGNABLE => 0
# };
#
1;
