#!/usr/bin/perl -w
use strict;
use blib;
use TQt;
use GameBoard;

TQt::Application::setColorSpec(&TQt::Application::CustomColor);
my $a = TQt::Application(\@ARGV);

my $gb = GameBoard;
$gb->setGeometry(100, 100, 500, 355);
$a->setMainWidget($gb);
$gb->show;
exit $a->exec;
